@@ -48,6 +48,7 @@
 #define LIBSHOUT_DEFAULT_HOST       "localhost"
 #define LIBSHOUT_DEFAULT_PORT       8000
 #define LIBSHOUT_DEFAULT_FORMAT     SHOUT_FORMAT_OGG
+#define LIBSHOUT_DEFAULT_USAGE      SHOUT_USAGE_UNKNOWN
 #define LIBSHOUT_DEFAULT_PROTOCOL   SHOUT_PROTOCOL_HTTP
 #define LIBSHOUT_DEFAULT_USER       "source"
 #define LIBSHOUT_DEFAULT_USERAGENT  "libshout/" VERSION
@@ -193,6 +194,10 @@ struct shout_connection_tag {
     shout_queue_t  rqueue;
     shout_queue_t  wqueue;
 
+    uint64_t       wait_timeout;
+    /* maybe we want to convert this to general flag vector later */
+    int            wait_timeout_happened;
+
     /* server capabilities (LIBSHOUT_CAP_*) */
     uint32_t server_caps;
 
@@ -210,6 +215,7 @@ struct shout {
     unsigned int protocol;
     /* type of data being sent */
     unsigned int format;
+    unsigned int usage;
     /* audio encoding parameters */
     util_dict *audio_info;
 
@@ -259,6 +265,8 @@ struct shout {
 };
 
 /* helper functions */
+const char *shout_get_mimetype_from_self(shout_t *self);
+
 int     shout_queue_data(shout_queue_t *queue, const unsigned char *data, size_t len);
 int     shout_queue_str(shout_connection_t *self, const char *str);
 int     shout_queue_printf(shout_connection_t *self, const char *fmt, ...);
@@ -279,7 +287,8 @@ int                 shout_connection_unref(shout_connection_t *con);
 int                 shout_connection_iter(shout_connection_t *con, shout_t *shout);
 int                 shout_connection_select_tlsmode(shout_connection_t *con, int tlsmode);
 int                 shout_connection_set_nonblocking(shout_connection_t *con, unsigned int nonblocking);
-int                 shout_connection_set_next_timeout(shout_connection_t *con, shout_t *shout, uint32_t timeout /* [ms] */);
+int                 shout_connection_set_wait_timeout(shout_connection_t *con, shout_t *shout, uint64_t timeout /* [ms] */);
+int                 shout_connection_get_wait_timeout_happened(shout_connection_t *con, shout_t *shout); /* returns SHOUTERR_* or > 0 for true */
 int                 shout_connection_connect(shout_connection_t *con, shout_t *shout);
 int                 shout_connection_disconnect(shout_connection_t *con);
 ssize_t             shout_connection_send(shout_connection_t *con, shout_t *shout, const void *buf, size_t len);
@@ -301,6 +310,7 @@ ssize_t      shout_tls_read(shout_tls_t *tls, void *buf, size_t len);
 ssize_t      shout_tls_write(shout_tls_t *tls, const void *buf, size_t len);
 int          shout_tls_recoverable(shout_tls_t *tls);
 int          shout_tls_get_peer_certificate(shout_tls_t *tls, char **buf);
+int          shout_tls_get_peer_certificate_chain(shout_tls_t *tls, char **buf);
 int          shout_tls_set_callback(shout_tls_t *tls, shout_tls_callback_t callback, void *userdata);
 #endif
 
